class SORTED_ARRAY [G -> COMPARABLE]
   -- for a sorted array

--------------------------------------------------------------------
creation {ANY}
   make

--------------------------------------------------------------------
feature {NONE}

   make (my_size: INTEGER) is
      require
	 size > -1
      do
	 size := my_size
	 !!elements.make (1, size)
	 upper := 0
      ensure
	 not (empty)
	 elements /= Void
      end  -- make

--------------------------------------------------------------------
feature {ANY}

   put (v: G) is
      require
	 not_full: not (full)
      local
	 index : INTEGER
	 stop : BOOLEAN
      do
	 from
	    index := upper
	    upper := upper + 1
	    stop := False
	 variant
	    index
	 until
	    stop or else (index < lower)
	 loop
	    if (v < elements.item (index)) then
	       elements.put (elements.item (index), index + 1)
	    else
	       elements.put (v, index + 1)
	       stop := True
	    end  -- if
	    index := index - 1
	 end  -- loop
	    
	 if (not (stop)) then
	    elements.put (v, 1)
	 end  -- if

      ensure
	 not (empty)
	 upper = old upper + 1
      end  -- put

--------------------------------------------------------------------
   item (i: INTEGER) : G is
      require
	 index_in_bounds: 0 < i and then i <= upper
      do
	 Result := elements.item (i)
      end  -- item

--------------------------------------------------------------------
   find (v: G) : BOOLEAN is
	 -- finds an element using a binary search algorithm 
      require
	 not_empty: not (empty)
      local
	 bottom, top, mid : INTEGER
      do
	 from
	    bottom := lower
	    top := upper
	    last_index := (bottom + top) // 2
	    Result := v.is_equal (elements.item (last_index))
	 invariant
	    bottom <= top
	 variant
	    top - bottom
	 until
	    Result or else (bottom = top)
	 loop
	    mid := (bottom + top) // 2
	    if (v.is_equal (elements.item (mid))) then
	       Result := True
	       last_index := mid
	    else
	       if (elements.item (mid) < v) then
		  bottom := mid + 1
	       else
		  top := mid - 1
	       end  -- if
	    end  -- if
	 end  -- loop

	 if (Result = False) then
	    -- item not found => last_index does not point to an entry
	    last_index := 0
	 end  -- if
	    
      ensure
         (Result = True) implies v.is_equal (elements.item (last_index))
      end  -- find

--------------------------------------------------------------------
   full : BOOLEAN is
      do
	 Result := (upper = size + 1)
      end  -- full

--------------------------------------------------------------------
   empty : BOOLEAN is
      do
	 Result := (upper < lower)
      end  -- empty

--------------------------------------------------------------------
   size : INTEGER
   last_index : INTEGER
   upper : INTEGER

--------------------------------------------------------------------
feature {NONE}

   -- private instance variables
   elements : ARRAY [G]
   lower : INTEGER is 1

--------------------------------------------------------------------
end -- class SORTED_ARRAY
