
----------------------------------------------------------
-- Towers of Hanoi -- GENERAL Second solution		--
----------------------------------------------------------

class   Pole

-----------------------------------------------------------
creation {ANY}
    make

-----------------------------------------------------------
feature {NONE}

-----------------------------------------------------------
    io : BASIC_IO
    pile : STACK[Ring]

-----------------------------------------------------------
feature {ANY}

-----------------------------------------------------------
    poleNo : INTEGER -- 1 to 3
    bigNum : INTEGER is 1000000	-- size of "top ring" when empty

-----------------------------------------------------------
    rightPole : Pole
    leftPole : Pole

-----------------------------------------------------------
    make (n : INTEGER) is
	require
	    num_range : (n >= 1) and (n <= 3)
        do
            !!io
	    !!pile.make
	    poleNo := n
	    io.put_string ("Creating Pole ")
	    io.put_int (poleNo)
	    io.put_string ("%N")
        end

-----------------------------------------------------------
    -- This must be called after all three poles are made
    linkPoles (left, right : Pole) is
        do
	    leftPole := left
	    rightPole := right
        end

-----------------------------------------------------------
    -- add a ring to the top of the pile
    -- but only if it is smaller than the top ring
    addRing (ring : Ring) is
	require
	    -- an exception will only be raised if the
	    -- implementation is incorrect
	    maintain_order : ring.size < topSize
        do
	    pile.add(ring)	-- push onto the stack
	    io.put_string("Moved Ring ")
	    io.put_int (ring.size)
	    io.put_string(" onto Pole ")
	    io.put_int (poleNo)
	    io.put_string ("%N")
        end

-----------------------------------------------------------
    -- tell the top ring to move
    moveRing is
	require
	    -- can't move a ring if the pole is empty
	    -- such a request should never be received
	    not_empty : not(isEmpty)
	local
	    top : Ring
	do
	    -- remove the top ring, and tell it to move itself
	    -- to another pole
	    top := pile.item
	    pile.remove
	    top.move
        end

-----------------------------------------------------------
    -- size of the top ring; a BIG number if the pole is empty
    -- (any ring can be put on an empty pole, so this is
    -- equivalent to an inifinitely big ring being on the pole)
    topSize : INTEGER is
	do
	    if pile.empty
	    then
	    	Result := bigNum
	    else
	        Result := pile.item.size
	    end
	end

-----------------------------------------------------------
    isEmpty : BOOLEAN is
	do
	    Result := pile.empty
	end

-----------------------------------------------------------
end -- class Pole

