--------------------------------------------------------------------
--      Eiffel/S Window System
--
--      %M%
--      Version %I%
--      Copyright (C) by Jean-Guy Schneider <schneidr@iam.unibe.ch>
--
--      Created:       30.6.1995
--      Last Modified: %G%
--------------------------------------------------------------------
class BALL  -- ball of the game
   
--------------------------------------------------------------------
creation {GAME}
   make
   
--------------------------------------------------------------------
feature {NONE}
   
   make (the_window : WINDOW, the_game : GAME, x_res, y_res, rad : INTEGER,
	 the_color : STRING) is
      require
	 non_void_window  : the_window /= Void
	 non_void_color   : the_color /= Void
	 pos_radius       : rad > 0
	 valid_resolution : x_res > 0 and then y_res > 0
      do
	 my_game := the_game
	 x_pos := (x_res / 4) 
	 y_pos := y_res / 2
	 radius := rad
	 !!my_ball.make (the_window, x_pos, y_pos, rad, the_color, True)
	 
	 -- set initial speed
	 x_speed := 3
	 y_speed := 2
      end  -- make

--------------------------------------------------------------------
feature {GAME}
   
   move is
      do
	 -- does the ball bounce somewhere?
	 if (my_game.bounce (x_pos+x_speed+sig_radius (x_speed), y_pos)) then
	    x_speed := -x_speed
	 end  -- if
	 if (my_game.bounce (x_pos, y_pos+y_speed+sig_radius (y_speed))) then
	    y_speed := -y_speed
	 end  -- if
	 
	 -- calculate new coordinates
	 x_pos := x_pos + x_speed
	 y_pos := y_pos + y_speed
	 
	 -- move the ball
	 my_ball.move (x_speed, y_speed)
      ensure
	 x_pos = old x_pos + x_speed
	 y_pos = old y_pos + y_speed
      end  -- move
   
--------------------------------------------------------------------
   in_game (x_res, y_res : INTEGER) : BOOLEAN is
      require
	 valid_resolution : x_res > 0 and then y_res > 0
      do
	 Result := ((x_pos >= 0) and then (x_pos <= x_res)) and then
	           ((y_pos >= 0) and then (y_pos <= y_res))
      end  -- in_game
      
--------------------------------------------------------------------
   remove is
      do
	 my_ball.remove
      end  -- remove

--------------------------------------------------------------------
feature {NONE}
   
   my_game          : GAME
   my_ball          : CIRCLE
   x_speed, y_speed : INTEGER
   x_pos, y_pos     : INTEGER
   radius           : INTEGER
	 
--------------------------------------------------------------------
   sig_radius (speed : INTEGER) : INTEGER is
      do
	 if (speed > 0) then
	    Result := radius
	 else
	    Result := -radius
	 end  -- if
      end  -- sig_radius

--------------------------------------------------------------------
invariant
   non_void_game   : my_game /= Void
   non_void_ball   : my_ball /= Void
   valid_radius    : radius > 0
   
--------------------------------------------------------------------
end  -- class BALL
