--------------------------------------------------------------------
--      Eiffel/S Window System
--
--      %M%
--      Version %I%
--      Copyright (C) by Jean-Guy Schneider <schneidr@iam.unibe.ch>
--
--      Created:       30.6.1995
--      Last Modified: %G%
--------------------------------------------------------------------
class PLAYER  -- the player of the game
   
--------------------------------------------------------------------
creation {GAME}
   make

--------------------------------------------------------------------
feature {NONE}
   
   make (the_window : WINDOW, x_res, y_res, width : INTEGER,
	 the_color : STRING) is
      require
	 non_void_window  : the_window /= Void
	 non_void_color   : the_color /= Void
	 valid_width      : width > 0 and then width < x_res
	                    and then width < y_res
	 valid_resolution : x_res > 0 and then y_res > 0
      do
	 -- init player
	 xRes := x_res
	 yRes := y_res
	 my_window := the_window
	 !!the_box.make (the_window, xRes-width, (yRes-2*width)/2, xRes,
			 (yRes+2*width)/2, the_color, True)
	 
	 -- set initial speed
	 x_speed := 0
	 y_speed := 5
      end  -- make
   
--------------------------------------------------------------------
feature {GAME}
   
   move is
      local
	 in_char : CHARACTER
	 y_dis   : INTEGER
      do
	 in_char := my_window.get_char
	 inspect in_char
	    when 'u' then y_dis := y_speed
	    when 'U' then y_dis := y_speed
	    when 'd' then y_dis := -y_speed
	    when 'D' then y_dis := -y_speed
	    else y_dis := 0
	 end  -- inspect
	 
	 if (y_dis /= 0) and then
	    ((the_box.bounding_box.bottom_left_y+y_dis) >= 0) and then
	    ((the_box.bounding_box.upper_right_y+y_dis) <= yRes) then
	    the_box.move (0, y_dis)
	 end  -- if
      end  -- move
   
--------------------------------------------------------------------
   bounce (x, y : INTEGER) : BOOLEAN is
      local
	 point_box : BOX
      do
	 !!point_box.make (x, y, x, y)
	 Result := point_box.intersect (the_box.bounding_box)
      end  -- bounce

--------------------------------------------------------------------
feature {NONE}
   
   my_window  : WINDOW
   the_box    : RECTANGLE
   x_speed    : INTEGER
   y_speed    : INTEGER
   xRes, yRes : INTEGER
   
--------------------------------------------------------------------
invariant
   non_void_box    : the_box /= Void
   non_void_window : my_window /= Void
   
--------------------------------------------------------------------
end  -- class PALYER
